﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment.Algorithms.Evolutionary {
  public class ESGQAPSolution : GQAPSolution {

    [Storable]
    internal DoubleValue sParam;
    public double SParam {
      get { return sParam.Value; }
      set {
        if (sParam.Value == value) return;
        sParam.Value = value;
        OnPropertyChanged(nameof(SParam));
      }
    }

    [StorableConstructor]
    protected ESGQAPSolution(bool deserializing) : base(deserializing) { }
    protected ESGQAPSolution(ESGQAPSolution original, Cloner cloner)
    : base(original, cloner) {
      sParam = cloner.Clone(original.sParam);
    }
    public ESGQAPSolution(IntegerVector assignment, Evaluation eval, double sParam)
      : base(assignment, eval) {
      this.sParam = new DoubleValue(sParam);
    }
    
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ESGQAPSolution(this, cloner);
    }
  }
}
