﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment.Algorithms.GRASP {
  [Item("GRASP+PR (GQAP) Context", "Context for GRASP+PR (GQAP).")]
  [StorableClass]
  public sealed class GRASPContext : PopulationContext<ISingleObjectiveSolutionScope<GQAPSolution>> {
    
    [Storable]
    private IValueParameter<GQAP> problem;
    public GQAP Problem {
      get { return problem.Value; }
      set { problem.Value = value; }
    }

    [Storable]
    private IValueParameter<GQAPSolution> bestSolution;
    public GQAPSolution BestSolution {
      get { return bestSolution.Value; }
      set { bestSolution.Value = value; }
    }
    
    public void SortPopulation() {
      Scope.SubScopes.Replace(Scope.SubScopes.OfType<ISingleObjectiveSolutionScope<GQAPSolution>>().OrderBy(x => Problem.Maximization ? -x.Fitness : x.Fitness).ToList());
    }
    
    [StorableConstructor]
    private GRASPContext(bool deserializing) : base(deserializing) { }
    private GRASPContext(GRASPContext original, Cloner cloner)
      : base(original, cloner) {
      problem = cloner.Clone(original.problem);
      bestSolution = cloner.Clone(original.bestSolution);
    }
    public GRASPContext() : this("GRASP+PR (GQAP) Context") { }
    public GRASPContext(string name) : base(name) {
      Parameters.Add(problem = new ValueParameter<GQAP>("Problem"));
      Parameters.Add(bestSolution = new ValueParameter<GQAPSolution>("BestFoundSolution"));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new GRASPContext(this, cloner);
    }

    public ISingleObjectiveSolutionScope<GQAPSolution> ToScope(GQAPSolution code, double fitness = double.NaN) {
      var name = Problem.Encoding.Name;
      var scope = new SingleObjectiveSolutionScope<GQAPSolution>(code,
        name + "Solution", fitness, Problem.Evaluator.QualityParameter.ActualName) {
        Parent = Scope
      };
      scope.Variables.Add(new Variable(name, code.Assignment));
      scope.Variables.Add(new Variable("Evaluation", code.Evaluation));
      return scope;
    }
  }
}
